/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */





    /// https://drafts.csswg.org/css-text-decor-3/#text-emphasis-property
    pub mod text_emphasis {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub text_emphasis_style:
                        Box<
                    longhands::text_emphasis_style::SpecifiedValue
                        >
                    ,
                pub text_emphasis_color:
                    longhands::text_emphasis_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub text_emphasis_style:
                    &'a longhands::text_emphasis_style::SpecifiedValue,
                pub text_emphasis_color:
                    &'a longhands::text_emphasis_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut text_emphasis_style = None;
                    let mut text_emphasis_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::TextEmphasisStyle(ref value) => {
                                text_emphasis_style = Some(value)
                            },
                            PropertyDeclaration::TextEmphasisColor(ref value) => {
                                text_emphasis_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        text_emphasis_style,
                        text_emphasis_color,
                ) {

                    (
                        Some(text_emphasis_style),
                        Some(text_emphasis_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            text_emphasis_style,
                            text_emphasis_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::TextEmphasisStyle(
                        longhands.text_emphasis_style
                    ));
                    declarations.push(PropertyDeclaration::TextEmphasisColor(
                        longhands.text_emphasis_color
                    ));
            })
        }

        
    use crate::properties::longhands::{text_emphasis_color, text_emphasis_style};

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let mut color = None;
        let mut style = None;

        loop {
            if color.is_none() {
                if let Ok(value) = input.try(|input| text_emphasis_color::parse(context, input)) {
                    color = Some(value);
                    continue
                }
            }
            if style.is_none() {
                if let Ok(value) = input.try(|input| text_emphasis_style::parse(context, input)) {
                    style = Some(value);
                    continue
                }
            }
            break
        }
        if color.is_some() || style.is_some() {
            Ok(expanded! {
                text_emphasis_color: unwrap_or_initial!(text_emphasis_color, color),
                text_emphasis_style: unwrap_or_initial!(text_emphasis_style, style),
            })
        } else {
            Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
        }
    }

    }


// CSS Compatibility
// https://compat.spec.whatwg.org/


    /// https://compat.spec.whatwg.org/#the-webkit-text-stroke
    pub mod _webkit_text_stroke {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub _webkit_text_stroke_width:
                    longhands::_webkit_text_stroke_width::SpecifiedValue
                    ,
                pub _webkit_text_stroke_color:
                    longhands::_webkit_text_stroke_color::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        #[derive(ToCss)]
        pub struct LonghandsToSerialize<'a> {
                pub _webkit_text_stroke_width:
                    &'a longhands::_webkit_text_stroke_width::SpecifiedValue,
                pub _webkit_text_stroke_color:
                    &'a longhands::_webkit_text_stroke_color::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut _webkit_text_stroke_width = None;
                    let mut _webkit_text_stroke_color = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::WebkitTextStrokeWidth(ref value) => {
                                _webkit_text_stroke_width = Some(value)
                            },
                            PropertyDeclaration::WebkitTextStrokeColor(ref value) => {
                                _webkit_text_stroke_color = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        _webkit_text_stroke_width,
                        _webkit_text_stroke_color,
                ) {

                    (
                        Some(_webkit_text_stroke_width),
                        Some(_webkit_text_stroke_color),
                    ) =>
                    Ok(LonghandsToSerialize {
                            _webkit_text_stroke_width,
                            _webkit_text_stroke_color,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::WebkitTextStrokeWidth(
                        longhands._webkit_text_stroke_width
                    ));
                    declarations.push(PropertyDeclaration::WebkitTextStrokeColor(
                        longhands._webkit_text_stroke_color
                    ));
            })
        }

        
    use crate::properties::longhands::{_webkit_text_stroke_color, _webkit_text_stroke_width};

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let mut color = None;
        let mut width = None;
        loop {
            if color.is_none() {
                if let Ok(value) = input.try(|input| _webkit_text_stroke_color::parse(context, input)) {
                    color = Some(value);
                    continue
                }
            }

            if width.is_none() {
                if let Ok(value) = input.try(|input| _webkit_text_stroke_width::parse(context, input)) {
                    width = Some(value);
                    continue
                }
            }
            break
        }

        if color.is_some() || width.is_some() {
            Ok(expanded! {
                _webkit_text_stroke_color: unwrap_or_initial!(_webkit_text_stroke_color, color),
                _webkit_text_stroke_width: unwrap_or_initial!(_webkit_text_stroke_width, width),
            })
        } else {
            Err(input.new_custom_error(StyleParseErrorKind::UnspecifiedError))
        }
    }

    }

