/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/. */




    
    

    /// https://drafts.csswg.org/css-box/#propdef-margin
    pub mod margin {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub margin_top:
                    longhands::margin_top::SpecifiedValue
                    ,
                pub margin_right:
                    longhands::margin_right::SpecifiedValue
                    ,
                pub margin_bottom:
                    longhands::margin_bottom::SpecifiedValue
                    ,
                pub margin_left:
                    longhands::margin_left::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub margin_top:
                    &'a longhands::margin_top::SpecifiedValue,
                pub margin_right:
                    &'a longhands::margin_right::SpecifiedValue,
                pub margin_bottom:
                    &'a longhands::margin_bottom::SpecifiedValue,
                pub margin_left:
                    &'a longhands::margin_left::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut margin_top = None;
                    let mut margin_right = None;
                    let mut margin_bottom = None;
                    let mut margin_left = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MarginTop(ref value) => {
                                margin_top = Some(value)
                            },
                            PropertyDeclaration::MarginRight(ref value) => {
                                margin_right = Some(value)
                            },
                            PropertyDeclaration::MarginBottom(ref value) => {
                                margin_bottom = Some(value)
                            },
                            PropertyDeclaration::MarginLeft(ref value) => {
                                margin_left = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        margin_top,
                        margin_right,
                        margin_bottom,
                        margin_left,
                ) {

                    (
                        Some(margin_top),
                        Some(margin_right),
                        Some(margin_bottom),
                        Some(margin_left),
                    ) =>
                    Ok(LonghandsToSerialize {
                            margin_top,
                            margin_right,
                            margin_bottom,
                            margin_left,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MarginTop(
                        longhands.margin_top
                    ));
                    declarations.push(PropertyDeclaration::MarginRight(
                        longhands.margin_right
                    ));
                    declarations.push(PropertyDeclaration::MarginBottom(
                        longhands.margin_bottom
                    ));
                    declarations.push(PropertyDeclaration::MarginLeft(
                        longhands.margin_left
                    ));
            })
        }

        
        #[allow(unused_imports)]
        use crate::parser::Parse;
        use crate::values::generics::rect::Rect;
        use crate::values::specified;

        pub fn parse_value<'i, 't>(
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |_c, i| {
                specified::LengthPercentageOrAuto::parse_quirky(_c, i, specified::AllowQuirks::Yes)
            })?;
            Ok(expanded! {
                    margin_top: rect.0,
                    margin_right: rect.1,
                    margin_bottom: rect.2,
                    margin_left: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
            where
                W: Write,
            {
                let rect = Rect::new(
                    &self.margin_top,
                    &self.margin_right,
                    &self.margin_bottom,
                    &self.margin_left,
                );
                rect.to_css(dest)
            }
        }
    
    }






    /// https://drafts.csswg.org/css-logical/#propdef-margin-block
    pub mod margin_block {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub margin_block_start:
                    longhands::margin_block_start::SpecifiedValue
                    ,
                pub margin_block_end:
                    longhands::margin_block_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub margin_block_start:
                    &'a longhands::margin_block_start::SpecifiedValue,
                pub margin_block_end:
                    &'a longhands::margin_block_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut margin_block_start = None;
                    let mut margin_block_end = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MarginBlockStart(ref value) => {
                                margin_block_start = Some(value)
                            },
                            PropertyDeclaration::MarginBlockEnd(ref value) => {
                                margin_block_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        margin_block_start,
                        margin_block_end,
                ) {

                    (
                        Some(margin_block_start),
                        Some(margin_block_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            margin_block_start,
                            margin_block_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MarginBlockStart(
                        longhands.margin_block_start
                    ));
                    declarations.push(PropertyDeclaration::MarginBlockEnd(
                        longhands.margin_block_end
                    ));
            })
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    use crate::values::specified;

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |_c: &ParserContext, input: &mut Parser<'i, 't>| {
            specified::LengthPercentageOrAuto::parse(_c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            margin_block_start: first,
            margin_block_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.margin_block_start;
            let second = &self.margin_block_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }






    /// https://drafts.csswg.org/css-logical/#propdef-margin-inline
    pub mod margin_inline {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub margin_inline_start:
                    longhands::margin_inline_start::SpecifiedValue
                    ,
                pub margin_inline_end:
                    longhands::margin_inline_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub margin_inline_start:
                    &'a longhands::margin_inline_start::SpecifiedValue,
                pub margin_inline_end:
                    &'a longhands::margin_inline_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut margin_inline_start = None;
                    let mut margin_inline_end = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::MarginInlineStart(ref value) => {
                                margin_inline_start = Some(value)
                            },
                            PropertyDeclaration::MarginInlineEnd(ref value) => {
                                margin_inline_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        margin_inline_start,
                        margin_inline_end,
                ) {

                    (
                        Some(margin_inline_start),
                        Some(margin_inline_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            margin_inline_start,
                            margin_inline_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::MarginInlineStart(
                        longhands.margin_inline_start
                    ));
                    declarations.push(PropertyDeclaration::MarginInlineEnd(
                        longhands.margin_inline_end
                    ));
            })
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    use crate::values::specified;

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |_c: &ParserContext, input: &mut Parser<'i, 't>| {
            specified::LengthPercentageOrAuto::parse(_c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            margin_inline_start: first,
            margin_inline_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.margin_inline_start;
            let second = &self.margin_inline_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }




    
    

    /// https://drafts.csswg.org/css-scroll-snap-1/#propdef-scroll-margin
    pub mod scroll_margin {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub scroll_margin_top:
                    longhands::scroll_margin_top::SpecifiedValue
                    ,
                pub scroll_margin_right:
                    longhands::scroll_margin_right::SpecifiedValue
                    ,
                pub scroll_margin_bottom:
                    longhands::scroll_margin_bottom::SpecifiedValue
                    ,
                pub scroll_margin_left:
                    longhands::scroll_margin_left::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub scroll_margin_top:
                    &'a longhands::scroll_margin_top::SpecifiedValue,
                pub scroll_margin_right:
                    &'a longhands::scroll_margin_right::SpecifiedValue,
                pub scroll_margin_bottom:
                    &'a longhands::scroll_margin_bottom::SpecifiedValue,
                pub scroll_margin_left:
                    &'a longhands::scroll_margin_left::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut scroll_margin_top = None;
                    let mut scroll_margin_right = None;
                    let mut scroll_margin_bottom = None;
                    let mut scroll_margin_left = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::ScrollMarginTop(ref value) => {
                                scroll_margin_top = Some(value)
                            },
                            PropertyDeclaration::ScrollMarginRight(ref value) => {
                                scroll_margin_right = Some(value)
                            },
                            PropertyDeclaration::ScrollMarginBottom(ref value) => {
                                scroll_margin_bottom = Some(value)
                            },
                            PropertyDeclaration::ScrollMarginLeft(ref value) => {
                                scroll_margin_left = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        scroll_margin_top,
                        scroll_margin_right,
                        scroll_margin_bottom,
                        scroll_margin_left,
                ) {

                    (
                        Some(scroll_margin_top),
                        Some(scroll_margin_right),
                        Some(scroll_margin_bottom),
                        Some(scroll_margin_left),
                    ) =>
                    Ok(LonghandsToSerialize {
                            scroll_margin_top,
                            scroll_margin_right,
                            scroll_margin_bottom,
                            scroll_margin_left,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ScrollMarginTop(
                        longhands.scroll_margin_top
                    ));
                    declarations.push(PropertyDeclaration::ScrollMarginRight(
                        longhands.scroll_margin_right
                    ));
                    declarations.push(PropertyDeclaration::ScrollMarginBottom(
                        longhands.scroll_margin_bottom
                    ));
                    declarations.push(PropertyDeclaration::ScrollMarginLeft(
                        longhands.scroll_margin_left
                    ));
            })
        }

        
        #[allow(unused_imports)]
        use crate::parser::Parse;
        use crate::values::generics::rect::Rect;
        use crate::values::specified;

        pub fn parse_value<'i, 't>(
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<Longhands, ParseError<'i>> {
            let rect = Rect::parse_with(context, input, |_c, i| {
                specified::Length::parse(_c, i)
            })?;
            Ok(expanded! {
                    scroll_margin_top: rect.0,
                    scroll_margin_right: rect.1,
                    scroll_margin_bottom: rect.2,
                    scroll_margin_left: rect.3,
            })
        }

        impl<'a> ToCss for LonghandsToSerialize<'a> {
            fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result
            where
                W: Write,
            {
                let rect = Rect::new(
                    &self.scroll_margin_top,
                    &self.scroll_margin_right,
                    &self.scroll_margin_bottom,
                    &self.scroll_margin_left,
                );
                rect.to_css(dest)
            }
        }
    
    }






    /// https://drafts.csswg.org/css-scroll-snap-1/#propdef-scroll-margin-block
    pub mod scroll_margin_block {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub scroll_margin_block_start:
                    longhands::scroll_margin_block_start::SpecifiedValue
                    ,
                pub scroll_margin_block_end:
                    longhands::scroll_margin_block_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub scroll_margin_block_start:
                    &'a longhands::scroll_margin_block_start::SpecifiedValue,
                pub scroll_margin_block_end:
                    &'a longhands::scroll_margin_block_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut scroll_margin_block_start = None;
                    let mut scroll_margin_block_end = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::ScrollMarginBlockStart(ref value) => {
                                scroll_margin_block_start = Some(value)
                            },
                            PropertyDeclaration::ScrollMarginBlockEnd(ref value) => {
                                scroll_margin_block_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        scroll_margin_block_start,
                        scroll_margin_block_end,
                ) {

                    (
                        Some(scroll_margin_block_start),
                        Some(scroll_margin_block_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            scroll_margin_block_start,
                            scroll_margin_block_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ScrollMarginBlockStart(
                        longhands.scroll_margin_block_start
                    ));
                    declarations.push(PropertyDeclaration::ScrollMarginBlockEnd(
                        longhands.scroll_margin_block_end
                    ));
            })
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    use crate::values::specified;

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |_c: &ParserContext, input: &mut Parser<'i, 't>| {
            specified::Length::parse(_c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            scroll_margin_block_start: first,
            scroll_margin_block_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.scroll_margin_block_start;
            let second = &self.scroll_margin_block_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }






    /// https://drafts.csswg.org/css-scroll-snap-1/#propdef-scroll-margin-inline
    pub mod scroll_margin_inline {
        use cssparser::Parser;
        use crate::parser::ParserContext;
        use crate::properties::{PropertyDeclaration, SourcePropertyDeclaration, MaybeBoxed, longhands};
        #[allow(unused_imports)]
        use selectors::parser::SelectorParseErrorKind;
        #[allow(unused_imports)]
        use std::fmt::{self, Write};
        #[allow(unused_imports)]
        use style_traits::{ParseError, StyleParseErrorKind};
        #[allow(unused_imports)]
        use style_traits::{CssWriter, KeywordsCollectFn, SpecifiedValueInfo, ToCss};

        #[derive(SpecifiedValueInfo)]
        pub struct Longhands {
                pub scroll_margin_inline_start:
                    longhands::scroll_margin_inline_start::SpecifiedValue
                    ,
                pub scroll_margin_inline_end:
                    longhands::scroll_margin_inline_end::SpecifiedValue
                    ,
        }

        /// Represents a serializable set of all of the longhand properties that
        /// correspond to a shorthand.
        pub struct LonghandsToSerialize<'a> {
                pub scroll_margin_inline_start:
                    &'a longhands::scroll_margin_inline_start::SpecifiedValue,
                pub scroll_margin_inline_end:
                    &'a longhands::scroll_margin_inline_end::SpecifiedValue,
        }

        impl<'a> LonghandsToSerialize<'a> {
            /// Tries to get a serializable set of longhands given a set of
            /// property declarations.
            pub fn from_iter<I>(iter: I) -> Result<Self, ()>
            where
                I: Iterator<Item=&'a PropertyDeclaration>,
            {
                // Define all of the expected variables that correspond to the shorthand
                    let mut scroll_margin_inline_start = None;
                    let mut scroll_margin_inline_end = None;

                // Attempt to assign the incoming declarations to the expected variables
                for longhand in iter {
                    match *longhand {
                            PropertyDeclaration::ScrollMarginInlineStart(ref value) => {
                                scroll_margin_inline_start = Some(value)
                            },
                            PropertyDeclaration::ScrollMarginInlineEnd(ref value) => {
                                scroll_margin_inline_end = Some(value)
                            },
                        _ => {}
                    };
                }

                // If any of the expected variables are missing, return an error
                match (
                        scroll_margin_inline_start,
                        scroll_margin_inline_end,
                ) {

                    (
                        Some(scroll_margin_inline_start),
                        Some(scroll_margin_inline_end),
                    ) =>
                    Ok(LonghandsToSerialize {
                            scroll_margin_inline_start,
                            scroll_margin_inline_end,
                    }),
                    _ => Err(())
                }
            }
        }

        /// Parse the given shorthand and fill the result into the
        /// `declarations` vector.
        pub fn parse_into<'i, 't>(
            declarations: &mut SourcePropertyDeclaration,
            context: &ParserContext,
            input: &mut Parser<'i, 't>,
        ) -> Result<(), ParseError<'i>> {
            #[allow(unused_imports)]
            use crate::properties::{NonCustomPropertyId, LonghandId};
            input.parse_entirely(|input| parse_value(context, input)).map(|longhands| {
                    declarations.push(PropertyDeclaration::ScrollMarginInlineStart(
                        longhands.scroll_margin_inline_start
                    ));
                    declarations.push(PropertyDeclaration::ScrollMarginInlineEnd(
                        longhands.scroll_margin_inline_end
                    ));
            })
        }

        
    #[allow(unused_imports)]
    use crate::parser::Parse;
    use crate::values::specified;

    pub fn parse_value<'i, 't>(
        context: &ParserContext,
        input: &mut Parser<'i, 't>,
    ) -> Result<Longhands, ParseError<'i>> {
        let parse_one = |_c: &ParserContext, input: &mut Parser<'i, 't>| {
            specified::Length::parse(_c, input)
        };

        let first = parse_one(context, input)?;
        let second =
            input.try(|input| parse_one(context, input)).unwrap_or_else(|_| first.clone());
        Ok(expanded! {
            scroll_margin_inline_start: first,
            scroll_margin_inline_end: second,
        })
    }

    impl<'a> ToCss for LonghandsToSerialize<'a>  {
        fn to_css<W>(&self, dest: &mut CssWriter<W>) -> fmt::Result where W: fmt::Write {
            let first = &self.scroll_margin_inline_start;
            let second = &self.scroll_margin_inline_end;

            first.to_css(dest)?;
            if first != second {
                dest.write_str(" ")?;
                second.to_css(dest)?;
            }
            Ok(())
        }
    }

    }


